// Emacs style mode select   -*- C++ -*- 
//-----------------------------------------------------------------------------
//
// $Id:$
//
// Copyright (C) 1993-1996 by id Software, Inc.
//
// This source is available for distribution and/or modification
// only under the terms of the DOOM Source Code License as
// published by id Software. All rights reserved.
//
// The source is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// FITNESS FOR A PARTICULAR PURPOSE. See the DOOM Source Code License
// for more details.
//
// $Log:$
//
// DESCRIPTION:
//	DOOM main program (D_DoomMain) and game loop (D_DoomLoop),
//	plus functions to determine game mode (shareware, registered),
//	parse command line parameters, configure game parameters (turbo),
//	and call the startup functions.
//
//-----------------------------------------------------------------------------


//static const char rcsid[] = "$Id: d_main.c,v 1.8 1997/02/03 22:45:09 b1 Exp $";

#define	BGCOLOR		7
#define	FGCOLOR		8


#include <os.h>

#include "doomdef.h"
#include "doomstat.h"

#include "dstrings.h"


#include "z_zone.h"
#include "w_wad.h"
#include "v_video.h"

#include "f_finale.h"
#include "f_wipe.h"

#include "m_misc.h"
#include "m_menu.h"

#include "i_system.h"
#include "i_video.h"

#include "g_game.h"

#include "hu_stuff.h"
#include "wi_stuff.h"
#include "st_stuff.h"
#include "am_map.h"

#include "p_setup.h"
#include "r_local.h"


#include "d_main.h"

//
// D-DoomLoop()
// Not a globally visible function,
//  just included for source reference,
//  called by D_DoomMain, never exits.
// Manages timing and IO,
//  calls all ?_Responder, ?_Ticker, and ?_Drawer,
//  calls I_GetTime, I_StartFrame, and I_StartTic
//
void D_DoomLoop (void);


char*			wadfiles[MAXWADFILES];

char*			current_dir;

boolean			devparm;	// started game with -devparm
boolean         nomonsters;	// checkparm of -nomonsters
boolean         respawnparm;	// checkparm of -respawn
boolean         fastparm;	// checkparm of -fast

boolean         drone;

boolean			singletics = true;

boolean			fuck = false;


extern boolean	inhelpscreens;

skill_t			startskill;
int				startepisode;
int				startmap;
boolean			autostart;

FILE*			debugfile;

boolean			advancedemo;




char		wadfile[1024];		// primary wad file
char		mapdir[1024];       // directory of development maps
char		basedefault[1024];  // default file

void D_ProcessEvents (void);
void G_BuildTiccmd (ticcmd_t* cmd);
void D_DoAdvanceDemo (void);

//
// D_PostEvent
// Called by the I/O functions when input is detected
//
void D_PostEvent (event_t* ev)
{
	if (M_Responder (ev))
		return;               // menu ate the event
	G_Responder (ev);
}

//
// D_Display
//  draw current display, possibly wiping it from the previous
//

// wipegamestate can be set to -1 to force a wipe on the next draw
gamestate_t     wipegamestate = GS_DEMOSCREEN;
extern boolean	setsizeneeded;
extern int		showMessages;
void 			R_ExecuteSetViewSize (void);

void D_Display (void)
{
    static  boolean			viewactivestate = false;
    static  boolean			menuactivestate = false;
    static  boolean			inhelpscreensstate = false;
    static  boolean			fullscreen = false;
    static  gamestate_t		oldgamestate = -1;
    static  int				borderdrawcount;
    int						nowtime;
    int						tics;
    int						wipestart;
    int						y;
    boolean					done;
    boolean					wipe;
    boolean					redrawsbar;
		
    redrawsbar = false;
	
	// change the view size if needed
    if (setsizeneeded)
    {
		R_ExecuteSetViewSize ();
		oldgamestate = -1;                      // force background redraw
		borderdrawcount = 3;
    }

    // save the current screen if about to wipe
    if (gamestate != wipegamestate)
    {
		wipe = true;
		wipe_StartScreen(0, 0, SCREENWIDTH, SCREENHEIGHT);
    }
    else
		wipe = false;
	
    if (gamestate == GS_LEVEL && gametic)
		HU_Erase();
	
    // do buffered drawing
    switch (gamestate)
    {
      case GS_LEVEL:
		if (!gametic)
			break;
		if (automapactive)
			AM_Drawer ();
		if (wipe || (viewheight != 200 && fullscreen) )
			redrawsbar = true;
		if (inhelpscreensstate && !inhelpscreens)
			redrawsbar = true;              // just put away the help screen
		ST_Drawer (viewheight == 200, redrawsbar );
		fullscreen = viewheight == 200;
		break;

      case GS_INTERMISSION:
		WI_Drawer ();
		break;

      case GS_FINALE:
		F_Drawer ();
		break;

      case GS_DEMOSCREEN:
		D_PageDrawer ();
		break;
    }
	
    // draw the view directly
	if (gamestate == GS_LEVEL && !automapactive && gametic)
		R_RenderPlayerView (&players[0]);
	
    if (gamestate == GS_LEVEL && gametic)
		HU_Drawer ();

    menuactivestate = menuactive;
    viewactivestate = viewactive;
    inhelpscreensstate = inhelpscreens;
    oldgamestate = gamestate;
    
	// draw pause pic
    if (paused)
    {
		if (automapactive)
			y = 4;
		else
			y = viewwindowy+4;
		V_DrawPatchDirect(viewwindowx+(scaledviewwidth-68)/2, y ,0 ,W_CacheLumpName ("M_PAUSE", PU_CACHE) );
    }

    // menus go directly to the screen
    M_Drawer ();          // menu is drawn even on top of everything
	
    // normal update
    if (!wipe)
    {
		I_Flip ();              // page flip or blit buffer
		return;
    }
    
    // wipe update
    wipe_EndScreen(0, 0, SCREENWIDTH, SCREENHEIGHT);

    wipestart = I_GetTime () - 1;

    do
    {
		do
		{
			nowtime = I_GetTime ();
			tics = nowtime - wipestart;
		} while (tics<=0);
		wipestart = nowtime;
		done = wipe_ScreenWipe(wipe_Melt, SCREENWIDTH, SCREENHEIGHT, tics);
		M_Drawer ();							// menu is drawn even on top of wipes
		I_Flip ();								// page flip or blit buffer
    } while (!done);
	gamestate = oldgamestate;
	wipegamestate= oldgamestate;
}



//
//  D_DoomLoop
//
extern  boolean		demorecording;

void D_DoomLoop (void)
{
    if (demorecording)
		G_BeginRecording ();
	
    I_InitGraphics ();

    while (!fuck)
    {
		I_StartTic ();
		
		if (advancedemo)
			D_DoAdvanceDemo ();
		
		M_Ticker (); //Menu ticker
		
		G_Ticker (); //Game ticker
		
		gametic++;

		// Update display, next frame, with current state
		D_Display ();

    }
	I_ShutdownGraphics();
	free(current_dir);
	fclose(lumpinfo->handle);
	
	return;
}



//
//  DEMO LOOP
//
int             demosequence;
int             pagetic;
char           *pagename;


//
// D_PageTicker
// Handles timing for warped projection
//
void D_PageTicker (void)
{
    if (--pagetic < 0)
		D_AdvanceDemo ();
}



//
// D_PageDrawer
//
void D_PageDrawer (void)
{
    V_DrawPatch (0,0, 0, W_CacheLumpName(pagename, PU_CACHE));
}


//
// D_AdvanceDemo
// Called after each demo or intro demosequence finishes
//
void D_AdvanceDemo (void)
{
    advancedemo = true;
}


//
// This cycles through the demo sequences.
// FIXME - version dependend demo numbers?
//
 void D_DoAdvanceDemo (void)
{
	///HAX HAX HAX
	pagename = "TITLEPIC";
	V_DrawPatch (0,0, 0, W_CacheLumpName(pagename, PU_CACHE));
	gamestate= GS_LEVEL;
	G_DeferedInitNew(2,1,1);
	advancedemo= false;
	///HAX HAX HAX
	
    /* UNCOMMENT AFTER DEMO PLAYING WORKS!!!
	
	players[consoleplayer].playerstate = PST_LIVE;  // not reborn
    advancedemo = false;
    usergame = false;               // no save / end game here
    paused = false;
    gameaction = ga_nothing;

    if ( gamemode == retail )
      demosequence = (demosequence+1)%7;
    else
      demosequence = (demosequence+1)%6;
	
	
    switch (demosequence)
    {
		case 0:
			if ( gamemode == commercial )
				pagetic = 35 * 11;
			else
				pagetic = 170;
			gamestate = GS_DEMOSCREEN;
			pagename = "TITLEPIC";
			break;
		
		case 1:
			G_DeferedPlayDemo ("demo1");
			break;
		
		case 2:
			pagetic = 200;
			gamestate = GS_DEMOSCREEN;
			pagename = "CREDIT";
			break;
		
		case 3:
			G_DeferedPlayDemo ("demo2");
			break;
		
		case 4:
			gamestate = GS_DEMOSCREEN;
			if ( gamemode == commercial)
			{
				pagetic = 35 * 11;
				pagename = "TITLEPIC";
			}
			else
			{
				pagetic = 200;

				if ( gamemode == retail )
					pagename = "CREDIT";
				else
					pagename = "HELP2";
			}
			break;
		
		case 5:
			G_DeferedPlayDemo ("demo3");
			break;
			// THE DEFINITIVE DOOM Special Edition demo
		
		case 6:
			G_DeferedPlayDemo ("demo4");
			break;
    }*/
}



//
// D_StartTitle
//
void D_StartTitle (void)
{
    gameaction = ga_nothing;
    demosequence = -1;
    D_AdvanceDemo ();
}




//      print title for every printed line
char            title[128];



//
// D_AddFile
//
void D_AddFile (char *file)
{
    //int     numwadfiles;
    char    *newfile;
	
    /*for (numwadfiles = 0 ; wadfiles[numwadfiles] ; numwadfiles++) PORT DEBUG, WE'LL ONLY EVER HAVE 1 WAD FILE ANYWAY
	;*/

    newfile = malloc (strlen(file)+1);
	
    strcpy (newfile, file);
	
    wadfiles[0] = newfile;
}

//
// IdentifyVersion
// Checks availability of IWAD files by name,
// to determine whether registered/commercial features
// should be executed (notably loading PWAD's).
//
void IdentifyVersion (void)
{
	char*		doom1wad;
    char*		doomwad;
    char*		doomuwad;
    char*		doom2wad;

    char*		plutoniawad;
    char*		tntwad;
	
	// Commercial.
    doom2wad = malloc(strlen(current_dir)+9+4+1);
    sprintf(doom2wad, "%sdoom2.wad.tns", current_dir);
	
	plutoniawad = malloc(strlen(current_dir)+12+4+1);
    sprintf(plutoniawad, "%splutonia.wad.tns", current_dir);

    tntwad = malloc(strlen(current_dir)+9+4+1);
    sprintf(tntwad, "%stnt.wad.tns", current_dir);

    // Retail.
    doomuwad = malloc(strlen(current_dir)+8+4+1);
    sprintf(doomuwad, "%sdoomu.wad.tns", current_dir);
    
    // Registered.
    doomwad = malloc(strlen(current_dir)+8+4+1);
    sprintf(doomwad, "%sdoom.wad.tns", current_dir);
    
    // Shareware.
    doom1wad = malloc(strlen(current_dir)+9+4+1);
    sprintf(doom1wad, "%sdoom1.wad.tns", current_dir);
	
    if ( access_ok (doom2wad) )
    {
		gamemode = commercial;
		D_AddFile (doom2wad);
		goto cleanup;
    }

    if ( access_ok (plutoniawad) )
    {
		gamemode = commercial;
		D_AddFile (plutoniawad);
		goto cleanup;
    }

    if ( access_ok (tntwad) )
    {
		gamemode = commercial;
		D_AddFile (tntwad);
		goto cleanup;
    }

    if ( access_ok (doomuwad) )
    {
		gamemode = retail;
		D_AddFile (doomuwad);
		goto cleanup;
    }

    if ( access_ok (doomwad) )
    {
		gamemode = registered;
		D_AddFile (doomwad);
		goto cleanup;
    }

    if ( access_ok (doom1wad) )
    {
		gamemode = shareware;
		D_AddFile (doom1wad);
		goto cleanup;
    }

    printf("Game mode indeterminate.\n");
    gamemode = indetermined;
	
   cleanup:
	free(doom1wad);
	free(doomwad);
	free(doomuwad);
	free(doom2wad);
	free(plutoniawad);
	free(tntwad);
}

//
// D_DoomMain
//
void D_DoomMain (char* loc)
{	
	//Find the current directory
	I_GetMainDir(loc);
	
    IdentifyVersion ();
	
    modifiedgame = false;
	
    nomonsters = 0;
    respawnparm = 0;
    fastparm = 0;
    devparm = 0;

    switch ( gamemode )
    {
      case retail:
		sprintf (title,
			 "The Ultimate DOOM Startup v%i.%i",
			 VERSION/100,VERSION%100);
		break;
      case shareware:
		sprintf (title,
			 "DOOM Shareware Startup v%i.%i",
			 VERSION/100,VERSION%100);
		break;
      case registered:
		sprintf (title,
			 "DOOM Registered Startup v%i.%i",
			 VERSION/100,VERSION%100);
		break;
      case commercial:
		sprintf (title,
			 "DOOM 2: Hell on Earth v%i.%i",
			 VERSION/100,VERSION%100);
		break;
    
      default:
		sprintf (title,
			 "Public DOOM - v%i.%i",
			 VERSION/100,VERSION%100);
		break;
    }
	
    printf ("%s\n",title);
	
    // get skill / episode / map from parms // DON'T, LEFT IN UNTIL OPCHECK
    startskill = sk_medium;
    startepisode = 1;
    startmap = 1;
    autostart = false;
    
    // init subsystems
    printf ("V_Init: Allocate screens\n");
    V_Init ();
	
    printf ("Z_Init: Init zone memory allocation daemon \n");
    Z_Init ();

    printf ("W_Init: Init WADfiles\n");
	
	printf("Running from %s \n",current_dir);
	
    W_InitMultipleFiles (wadfiles);
    
	//Just in case we haven't found the WAD files...
	if (fuck)
		return;

    // Check for -file in shareware
    if (modifiedgame)
    {
		// These are the lumps that will be checked in IWAD,
		// if any one is not present, execution will be aborted.
		char name[23][8]=
		{
			"e2m1","e2m2","e2m3","e2m4","e2m5","e2m6","e2m7","e2m8","e2m9",
			"e3m1","e3m3","e3m3","e3m4","e3m5","e3m6","e3m7","e3m8","e3m9",
			"dphoof","bfgga0","heada1","cybra1","spida1d1"
		};
		int i;
		
		if ( gamemode == shareware)
			I_Error("You cannot -file with the shareware "
				"version. Register!");

		// Check for fake IWAD with right name,
		// but w/o all the lumps of the registered version. 
		if (gamemode == registered)
			for (i = 0;i < 23; i++)
				if (W_CheckNumForName(name[i])<0)
					I_Error("This is not the registered version.");
    }
    
    // Check and print which version is executed.
    switch ( gamemode )
    {
      case shareware:
      case indetermined:
		printf (
			"=====================================\n"
			"             Shareware!\n"
			"=====================================\n"
		);
		break;
      case registered:
      case retail:
      case commercial:
		printf (
			"=====================================\n"
			"          Commercial product\n"
			"          do not distribute!\n"
			"=====================================\n"
		);
		break;
	
      default:
		// Ouch.
		break;
    }
	
	printf ("I_Init: Setting up machine state\n");
    I_Init ();

    printf ("M_Init: Init miscellaneous info\n");
    M_Init ();

    printf ("R_Init: Init DOOM refresh daemon \n");
    R_Init ();
	
    printf ("P_Init: Init Playloop state\n");
    P_Init ();

    printf ("HU_Init: Setup heads up display\n");
    HU_Init ();
	
    printf ("ST_Init: Init status bar\n");
    ST_Init ();

	printf ("Engage... \n");
	D_StartTitle ();
	D_DoomLoop ();  // never returns, except when it does
	
	return;
}
